package edu.washington.math.sage.paintplay;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Random;
import java.util.Set;

import javax.swing.JPanel;

public class PaintPlayGUIDrawingPanel extends JPanel{
	PaintPlayMouseListener myPPSMouseListener;
	PaintPlayGUIPanel myParentGUIpanel;
	
	Image i;
    int width = 200;
    int height = 200;
    int num=0;
    double gravity=0;//1000;//500;//,100;
    double epsilon=0.1;
    double damping=0.3;
    double restitution=0.3;
    double idealdistance=1;//15;//20//10;
    BufferedImage bimage;
    Random generator = new Random();
    LinkedList<Particle>[][] pixels;
    Set<Particle> particles;
    
    public PaintPlayGUIDrawingPanel(PaintPlayGUIPanel theParentGUIpanel){
    	super();
        myParentGUIpanel=theParentGUIpanel;
        myPPSMouseListener =
            new PaintPlayMouseListener(myParentGUIpanel);
    	this.addMouseMotionListener(myPPSMouseListener);
        this.addMouseListener(myPPSMouseListener);
    	this.setBackground(Color.white);
    	this.setForeground(Color.blue);
    	this.setPreferredSize(new Dimension(500,500));
    	pixels=new LinkedList[width][height];
    	for(int x=0;x<width;x++){
        	for(int y=0;y<height;y++){
        		pixels[x][y]=new LinkedList<Particle>();
        	}
    	}
        particles=new HashSet<Particle>();
        bimage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
    }
    public void paintComponent(Graphics g){
    	super.paintComponent(g);
    	g.drawImage (bimage, 0, 0, this);
    }
    public double repellingforce(double distance){
    	if (distance<idealdistance){
    	  return 1000*(distance-idealdistance);//50,500,100
    	}else{
    		return 0;
        }
    }
    public double sqr(double x){
    	return x*x;
    }
    public void applyPhysics(double deltatime){
    	for (Particle particle:particles){
    		int oldx=(int) particle.oldx;
    		int oldy=(int) particle.oldy;
    		
    		//gravitational force
    		double yforce=gravity*particle.mass;
    		double xforce=0;
    		
    		//damping
    		yforce-=Math.signum(particle.oldvy)*particle.oldvy*particle.oldvy*damping;
    		xforce-=Math.signum(particle.oldvx)*particle.oldvx*particle.oldvx*damping;
    		
    		//repelling forces
    		for (int xcoord=Math.max(0,oldx-3);xcoord<=Math.min(oldx+3,width-1);xcoord++){
    			for (int ycoord=Math.max(0,oldy-3);ycoord<=Math.min(oldy+3,height-1);ycoord++){
    				for (Particle otherParticle:pixels[xcoord][ycoord]){
    					if (otherParticle==particle){
    						continue;//bodies don't exert forces on themselves
    					}
    					double xdiff=otherParticle.oldx-particle.oldx;
    					double ydiff=otherParticle.oldy-particle.oldy;
    					double distance = Math.sqrt(sqr(xdiff)+sqr(ydiff));
    					double totalforce=repellingforce(distance);
    					double theta=Math.atan2(ydiff, xdiff);//TODO:BOTTLENECK!!!!
    					yforce+=Math.sin(theta)*totalforce;
    					xforce+=Math.cos(theta)*totalforce;
    				}
    			}
    		}
    		
    		//penalty forces from walls
            //handle bottom wall
    		if (particle.oldy>height-10){
    			//penalty force
    			yforce-=(particle.oldy-height+10)*1;
    		}
    		//handle top wall
    		else if (particle.oldy<10){
    			//penalty force
    			yforce+=(particle.oldy+10)*1;
    		}
    		//handle left wall
    		if (particle.oldx<10){
    			//penalty force
    			xforce+=(particle.oldx+10)*1;
    		}
    		//handle right wall
    		else if (particle.oldx>width-10){
    			//penalty force
    			xforce-=(particle.oldx-width+10)*1;
    		}
    		
    		particle.vx=particle.oldvx+xforce/particle.mass*deltatime;
    		particle.vy=particle.oldvy+yforce/particle.mass*deltatime;
    		
    		//collisions with walls
            //handle bottom wall
    		if (particle.oldy>height-10){
    			if (particle.vy>0){
    				particle.vy*=-restitution;
    			}
    		}
    		//handle top wall
    		else if (particle.y<10){
    			if (particle.vy<0){
    				particle.vy*=-restitution;
    			}
    		}
    		//handle left wall
    		if (particle.x<10){
    			if (particle.vx<0){
    				particle.vx*=-restitution;
    			}
    		}
    		//handle right wall
    		else if (particle.x>width-10){
    			if (particle.vx>0){
    				particle.vx*=-restitution;
    			}
    		}
 
    		particle.x=particle.oldx+particle.vx*deltatime;
    		particle.y=particle.oldy+particle.vy*deltatime;

    		int x=(int) particle.x;
    		int y=(int) particle.y;
    		
    		//don't let particles leave box
    		if (x<0){
    			x=1;
    			particle.x=1;
    		}
    		if (x>=width){
    			x=width-2;
    			particle.x=width-2;
    		}
    		if (y<0){
    			y=1;
    			particle.y=1;
    		}
    		if (y>=height){
    			y=height-2;
    			particle.y=height-2;
    		}
    		
    	}
    }
    public void applynewstate(){
    	for (Particle particle:particles){
    		int x=(int) particle.x;
    		int y=(int) particle.y;
    		int oldx=(int) particle.oldx;
    		int oldy=(int) particle.oldy;
    		if (x!=oldx|y!=oldy){
    			pixels[oldx][oldy].remove(particle);
    			pixels[x][y].add(particle);
    		}
    		particle.oldy=particle.y;
    		particle.oldx=particle.x;
    		particle.oldvy=particle.vy;
    		particle.oldvx=particle.vx;
    	}
    }
    public void pointcreateimage(){
    	for(int x=0;x<width;x++){
        	for(int y=0;y<height;y++){
               if (pixels[x][y].size()>0){
            	   bimage.setRGB(x, y, pixels[x][y].get(0).color.getRGB());
               }else{
            	   bimage.setRGB(x, y, 0);
               }
        	}
    	}
    }
    public void averagecreateimage(){
    	for(int x=0;x<width;x++){
        	for(int y=0;y<height;y++){
        		
        		int red=255*10;
        		int green=255*10;
        		int blue=255*10;
        		int numparticles=10;
        		for (int xcoord=Math.max(0,x-5);xcoord<=Math.min(x+5,width-1);xcoord++){
        			for (int ycoord=Math.max(0,y-5);ycoord<=Math.min(y+5,height-1);ycoord++){
        				int manhattendistance=Math.abs(x-xcoord)+Math.abs(y-ycoord);
        				int weight=11-manhattendistance;
        				for (Particle particle:pixels[xcoord][ycoord]){
        					numparticles+=weight;
        					red+=(particle.color.getRed())*weight;
        					green+=(particle.color.getGreen())*weight;
        					blue+=(particle.color.getBlue())*weight;
        				}
        			}
        		}
        		
               if (numparticles>4){
            	   red/=numparticles;
           	       green/=numparticles;
           		   blue/=numparticles;
           		bimage.setRGB(x, y, red*65536+green*256+blue);
               }else{
            	   red=255;
            	   green=255;
            	   blue=255;
            	   bimage.setRGB(x, y, red*65536+green*256+blue);
               }
        	}
    	}
    }
    public void additivecreateimage(){
    	for(int x=0;x<width;x++){
        	for(int y=0;y<height;y++){
        		
        		int red=0;
        		int green=0;
        		int blue=0;
        		for (int xcoord=Math.max(0,x-5);xcoord<=Math.min(x+5,width-1);xcoord++){
        			for (int ycoord=Math.max(0,y-5);ycoord<=Math.min(y+5,height-1);ycoord++){
        				int manhattendistance=Math.abs(x-xcoord)+Math.abs(y-ycoord);
        				double weight=1-(1-(11-manhattendistance)/11.0)/2;
        				for (Particle particle:pixels[xcoord][ycoord]){
        					red=Math.max(red,(int)(particle.color.getRed()*weight));
        					green=Math.max(green,(int)(particle.color.getGreen()*weight));
        					blue=Math.max(blue,(int)(particle.color.getBlue()*weight));
        				}
        			}
        		}
           		bimage.setRGB(x, y, red*65536+green*256+blue);
        	}
    	}
    }
    public void subtractivecreateimage(){
    	for(int x=0;x<width;x++){
        	for(int y=0;y<height;y++){
        		
        		int red=255;
        		int green=255;
        		int blue=255;
        		for (int xcoord=Math.max(0,x-5);xcoord<=Math.min(x+5,width-1);xcoord++){
        			for (int ycoord=Math.max(0,y-5);ycoord<=Math.min(y+5,height-1);ycoord++){
        				int manhattendistance=Math.abs(x-xcoord)+Math.abs(y-ycoord);
        				double weight=(11-manhattendistance)/11.0;
        			    weight=1-(1-(11-manhattendistance)/11.0)/2;

        				for (Particle particle:pixels[xcoord][ycoord]){
        					red=255-Math.max(255-red,(int)((255-particle.color.getRed())*weight));
        					green=255-Math.max(255-green,(int)((255-particle.color.getGreen())*weight));
        					blue=255-Math.max(255-blue,(int)((255-particle.color.getBlue())*weight));
        				}
        			}
        		}
           		bimage.setRGB(x, y, red*65536+green*256+blue);
        	}
    	}
    }
    public void updateGraphics(double deltatime){
		num++;
		//System.out.println(num);
		myParentGUIpanel.mousex=Math.max(myParentGUIpanel.mousex,0);
		myParentGUIpanel.mousex=Math.min(myParentGUIpanel.mousex,width-1);
		myParentGUIpanel.mousey=Math.max(myParentGUIpanel.mousey,0);
		myParentGUIpanel.mousey=Math.min(myParentGUIpanel.mousey,height-1);
		//Create new paint
		if (myParentGUIpanel.mousePressed & myParentGUIpanel.mode=="Create")
		{
			myParentGUIpanel.mousex=Math.max(myParentGUIpanel.mousex,0);
			myParentGUIpanel.mousex=Math.min(myParentGUIpanel.mousex,width-1);
			myParentGUIpanel.mousey=Math.max(myParentGUIpanel.mousey,0);
			myParentGUIpanel.mousey=Math.min(myParentGUIpanel.mousey,height-1);
			int x=myParentGUIpanel.mousex;
			int y=myParentGUIpanel.mousey;

			for (int xcoord=Math.max(0,x-5);xcoord<=Math.min(x+5,width-1);xcoord+=2){
    			for (int ycoord=Math.max(0,y-5);ycoord<=Math.min(y+5,height-1);ycoord+=2){
    				double x2=xcoord+generator.nextDouble();
    			    double y2=ycoord+generator.nextDouble();
    				Particle particle = new Particle(x2,y2,generator.nextDouble()-0.5,generator.nextDouble()-0.5,myParentGUIpanel.drawingColor,1);
    				particles.add(particle);
    				pixels[(int)x2][(int)y2].add(particle);
    			}
    		}
		}
        applyPhysics(deltatime);
        applynewstate();
    }
}
